# NoMorePass DART libraries

This library allows you to use the nomorepass.com system to send and
receive credentials using nomorepass mobile application. It is intended 
to use in any environment, so it does not generate / print the qr-code, 
instead provides the text that should be included in the qrcode 
(you can generate using any qrcode libraries).

## Usage

To receive passwords:

```
// Nomorepass initialization
  Nomorepass nmp = Nomorepass();
  nmp.apikey = 'MYAPIKEY';

  // QR to send a password
  print("Generating QR...");
  print(await nmp.getQrText('test'));
  print("Draw and scan this QR with nomorepass to send a password back");
  Map? res = null;
  Future.delayed(const Duration(seconds: 30), () {
    if (res == null) {
      nmp.stopped = true;
    }
  }).then((value) {
    if (nmp.stopped == true) {
      print("Aborted!");
      nmp.stop();
    }
  });
  print("Waiting for reception (30sec waiting)");
  nmp.stopped = false;
  res = await nmp.start();
  if (res != null && res.containsKey('error')) {
    print("Error ${res['error']}");
  } else {
    if (res != null) {
      print("User: ${res['user']}");
      print("Pass: ${res['password']}");
      print("Extra: ${res['extra']}");
    }
  }
```

To send password:

```
  // Nomorepass initialization
  Nomorepass nmp = Nomorepass();
  nmp.apikey = 'MYAPIKEY';

  print("Generating QR");
  print(await nmp.getQrSend("Test password", "usertest", "mypassword", {}));
  print("Scan this QR with nomorepass to receive a new password");
  Map? res = null;
  Future.delayed(const Duration(seconds: 30), () {
    if (res == null) {
      nmp.stopped = true;
    }
  }).then((value) {
    if (nmp.stopped == true) {
      print("Aborted!");
      nmp.stop();
    }
  });
  print("Waiting for reception (30sec waiting)");
  res = await nmp.send();
  print ("Received!);
```

## Examples

You have a exampe file (example.dart) in the example file

## How to use NoMorePass

1. Download and install the mobile app

* [android] https://play.google.com/store/apps/details?id=com.biblioeteca.apps.NoMorePass
* [ios] https://itunes.apple.com/us/app/no-more-pass/id1199780162?l=es&ls=1&mt=8

3. Open it and create a new password (or use some of yours)
4. Then you can scan the qrcode generated by the library to send securely this password to your app or send/update passwords from your code to the app.

## Help / more info

Visit [nomorepass.com](https://nomorepass.com) or leave an Issue

(C) 2021 Nomorepass.com + Biblioeteca Technologies
